""" Survey

Some code taken from ATTopic in ATContentTypes.

Licensed under the ZPL.

The rest:
(C) 2004 by Dan Keshet (also under the ZPL)

"""
#from Products.ATContentTypes.types.ATContentType import ATCTOrderedFolder
#from Products.ATContentTypes.types.schemata import ATContentTypeSchema
from Products.Archetypes.public import OrderedBaseFolderSchema, OrderedBaseFolder
from Products.Archetypes.public import registerType

from Permissions import *
from AccessControl import ClassSecurityInfo
from BTrees.OIBTree import OIBTree

from questions import QuestionRegistry, BaseSurveyQuestion

schema = OrderedBaseFolderSchema.copy()
for i in ['id','title']:
	schema[i].schemata = 'metadata'

def modify_fti(fti):
	"""Initial view is our view script"""
	fti['immediate_view'] = 'base_metadata'


class Survey(OrderedBaseFolder):
	"""Surveys can be edited, taken, and analyzed within the Plone interface.  
	   They offer a variety of question types.
	"""
	schema = schema
	content_icon   = 'survey_icon.png'
	meta_type	  = 'Survey'
	archetype_name = 'Survey'
	use_folder_tabs = 0
	filter_content_types  = 1
	allowed_content_types = ()	
	security = ClassSecurityInfo()
	
	def __init__(self, id, **kwargs):
		self._users = OIBTree()
		self._pages = ['1']
		OrderedBaseFolder.__init__(self,id, **kwargs)
	
	def manage_afterClone(self, item):
		"""Get rid of the list of users who have taken the survey
		and start afresh"""
		self._users = OIBTree()
		OrderedBaseFolder.manage_afterClone(self, item)
		
	security.declareProtected(TAKE_SURVEY, 'getUniqueId')
	def getUniqueId(self, user, request=None):
		"""Returns a different id for each username (or browser id in case of anon.), 
		so that we can decide whether to keep track of who answered what or not"""
		if type(user) == type(''):
			userid = user
		else:
			try:
				userid = user.getId()				
			except AttributeError:
				raise AttributeError("Not a user %s" % user)
		if userid is None:
			 try:
				userid = 'Anonymous User (%s) ' % request.SESSION.getBrowserIdManager().getBrowserId()
			 except AttributeError:
				 userid = 'Anonymous User'
		try:
			l = self._users[userid]
 		except KeyError:
			l = len(self._users)
			self._users[userid] = l
		return l
		
	security.declareProtected(VIEW_RESULTS, 'listRespondents')
	def listRespondents(self):
		"""List available field types"""
		return list(self._users.keys())
	
	security.declareProtected(EDIT_CONTENT_PERMISSION, 'addQuestion')
	def addQuestion(self, field_type, page=None):
		"""Add a new question.
		"""
		if page is None:
			page = self.getPages()[0]
		newid = self.generateUniqueId(field_type)
		ft	= QuestionRegistry.get(field_type,None)
		if ft is None:
			raise ValueError, "%s is not a valid question type" % field_type
		question = ft(newid,schemata=page)
		self._setObject( newid, question)
		
	def listQuestionTypes(self):
		"""List available question types"""
		return QuestionRegistry.listTypes()
			
	def listQuestionTypeValues(self):
		"""List available question types with meta_types."""
		return QuestionRegistry.listMetaAndShort()
	
	def listQuestions(self, page=None, skiplabels=0):
		"""List current questions"""
		types=None
		if skiplabels:
			types = [x for x in self.listQuestionTypes() if x != 'LabelQuestion']
		values = self.contentValues(spec=types)
		if page is None:
			return values
		return [q for q in values if q.getLocalSchemata() == page]
	
	security.declareProtected(EDIT_CONTENT_PERMISSION, 'addPage')
	def addPage(self,cur_page):
		l = self.getPages()
		i = 1
		while i:
			if str(i) not in l:
				break
			i += 1
		i = str(i)
		try:
			idx = l.index(cur_page)
			l.insert(l.index(cur_page)+1,i)
		except ValueError:
			l.append(i)
		self._pages = l
		return i
		
	def delPage(self,page):
		"""Remove all questions on a particular page and the page itself"""
		self.manage_delObjects([x.id for x in self.listQuestions(page=page)])
		if len(self._pages) > 1:
			try:
				idx = self._pages.index(page) - 1
				self._pages.remove(page)
				self._pages = self._pages #or use the _p_changed magic.
				return self._pages[idx]
			except ValueError:
				return None
		
	def getPages(self):
		"""Return all pages"""
		return self._pages
	
	actions = (
		{
		'id'		  : 'view',
		'name'		  : 'View',
		'action'	  : 'string:${folder_url}/survey_view',
		'permissions' : (VIEW_CONTENT_PERMISSION,),
		'visible'     : 0
		},
		{
		'id'		  : 'edit',
		'name'		: 'Edit',
		'action'	  : 'string:${folder_url}/survey_questions',
		'permissions' : (EDIT_CONTENT_PERMISSION,)
		},
		{
		'id'		  : 'results',
		'name'		: 'Results',
		'action'	  : 'string:${folder_url}/survey_results',
		'permissions' : (VIEW_RESULTS,)
		},
		{
		'id'		  : 'take',
		'name'		: 'Take Survey',
		'action'	  : 'string:${folder_url}/survey_take',
		'permissions' : (TAKE_SURVEY,)
		}
	)

registerType(Survey)